:- use_module(library(system)).
:- use_module(library(lists)).
:- use_module(library(terms)).
:- use_module(library(charsio)).
:- use_module(quiz_data).

:- set_prolog_flag(language, iso).

ask(N) :-
	t(N, G, X, C),
	write_question(G, X),
	read(A),
	wrapped_call(G, X, RR),
	check_ans(A, RR, C).

compose_question(G0, X0, Goal, Var) :-
	copy_term(G0-X0, G-X), X = '$VAR'(23),
	numbervars(G, 0, _),
	with_output_to_chars(write_term(G, [quoted(true),numbervars(true)]),
			     Goal),
	with_output_to_chars(write_term(X, [quoted(true),numbervars(true)]),
			     Var).

wrapped_call(G, R) :-
	catch(G, _, R=exception), !, % No multiple solutions!
	(   var(R) -> R = success
	;   true
	).
wrapped_call(_, failure).

check_ans(success, AS, success, RS, _, R, P) :-
	atomic_term(AS, S),
	variant(S, RS), !,
	correct(R), P = proceed.
check_ans(exception, _, exception, _, _, R, P) :- !,
	correct(R), P = proceed.
check_ans(failure, _, failure, _, _, R, P) :- !,
	correct(R), P = proceed.
check_ans(_, _, _, _, _, R, P) :-
	incorrect(R), P = stay.

solution(N, R) :-
	t(N, G, RSub, C), !,
	wrapped_call(G, RSuc),
	question(N, q(_N, Q, _Var)),
	R = ['A helyes vlasz a(z)', pre(Q), 'krdsre ', ARS, RC],
	write_to_chars(RSub, RSubCs),
	compose_suc(RSuc, RSubCs, ARS),
	comment(RSuc, C, RC).

compose_suc(success,   Sub, [em(siker), ' s ', tt('X'), ' = ', tt(Sub)]).
compose_suc(failure,   _,   em(meghisuls)).
compose_suc(exception, _,   em(hiba)).

comment(success, _, RC) :- !,
        RC = '.'.
comment(_, C, [', mert ', C]).


correct('Helyes.').

incorrect('Helytelen.').

atomic_term(A, T) :-
	name(A, AC0),
	append(AC0, [0'.], AC),
	read_from_chars(AC, T).

question(stay, N, Q) :-
	question(N, Q).
question(proceed, N, Q) :-
	N1 is N+1,
	question(N1, Q).

question(N, Q) :-
	t(N, G, X, _), !, Q = q(CN,['| ?- ',Goal,'.'],Var),
	number_codes(N, CN),
	compose_question(G, X, Goal, Var).
question(_, end).

answer(_, Suc, _, R, P) :-
	form_empty_value(Suc), !,
	R = 'Hinyos megolds.',
	P = stay.
answer(_, success, Sub, R, P) :-
	form_empty_value(Sub), !,
	R = 'Hinyos megolds.',
	P = stay.
answer(N, Suc, Sub, R, P) :-
	t(N, G, RSub, C), !,
	wrapped_call(G, RSuc),
	check_ans(Suc, Sub, RSuc, RSub, C, R, P).
answer(_, _, _, 'Nincs ilyen sorszm feladat.', stay).

:- use_module(library(pillow)).

response(Input, Q, R) :-
	get_form_value(Input, number, Val),
	(   form_empty_value(Val) ->
	    question(1, Q), R = []
	;   get_form_value(Input, restart, 'jra') ->
	    question(1, Q), R = []
	;   get_form_value(Input, solve, 'Megold') ->
	    solution(Val, R),
	    question(proceed, Val, Q)
	;   get_form_value(Input, success, Suc),
	    get_form_value(Input, substitution, Sub),
	    answer(Val, Suc, Sub, R, P),
	    question(P, Val, Q)
	).

main:-
	get_form_input(Input),
	response(Input, Question, Response0),
	(  Response0 == [] -> Response = []
	;  Response = [Response0, --]
	),
	reply(Question, Response).

reply(q(Num, Goal, Var), Response) :-
	output_html([
		     cgi_reply,
		     start,
		     title('Prolog kvz'),
		     Response,
		     h3([Num, '. feladat']),
		     start_form,
		     input(hidden, [name=number,value=Num]),
		     'Mi lesz az albbi hvs eredmnye?',\\,
		     pre(Goal),\\,
		     radio(success,success,[]), ' Siker  ',
		     radio(success,failure,[]), ' Meghisuls  ',
		     radio(success,exception,[]), ' Hiba',$,
		     'Ha sikeres, akkor mi lesz ',
		     code(Var), ' rtke a hvs utn?',\\,
		     textinput(substitution, [size=30], ''),\\,
		     input(submit,[name=submit, value="Elkld"]),
		     input(reset,[value="Trl"]),
		     input(submit,[name=solve, value="Megold"]),
		     end_form,
		     end]).
reply(end, Response) :-
	output_html([
		     cgi_reply,
		     start,
		     title('Prolog kvz'),
		     Response,
		     start_form,
		     input(submit, [name=restart, value="jra"]),
		     end_form,
		     end]).

:-save_cgi('quiz1.cgi',user:main), halt.
